from __future__ import unicode_literals

import balanced
from django import forms
from django.conf.urls import patterns, url
from django.contrib import admin
from django.contrib.auth.models import User
from django.core import urlresolvers
from django.shortcuts import render, redirect

from django_balanced.models import BankAccount, Credit

"""
TODO:
    Generate merchant dashboard login links
    Bulk pay a set of bank accounts
    Add account URI onto django users
"""


class BalancedAdmin(admin.ModelAdmin):
    add_fields = ()
    edit_fields = ()

    def add_view(self, *args, **kwargs):
        self.fields = getattr(self, 'add_fields', self.fields)
        return super(BalancedAdmin, self).add_view(*args, **kwargs)

    def change_view(self, *args, **kwargs):
        self.fields = getattr(self, 'edit_fields', self.fields)
        return super(BalancedAdmin, self).change_view(*args, **kwargs)


class BankAccountAdminForm(forms.ModelForm):
    name = forms.CharField(max_length=255)
    account_number = forms.CharField(max_length=255)
    routing_number = forms.CharField(max_length=255)
    type = forms.ChoiceField(choices=(
        ('savings', 'savings'), ('checking', 'checking')
    ))
    user = forms.ModelChoiceField(queryset=User.objects, required=False)

    class Meta:
        model = BankAccount

#
#    def clean(self):
#        data = self.cleaned_data
#        # TODO: validate routing number
#        #        routing_number = balanced.BankAccount(
#        #            routing_number=data['routing_number'],
#        #        )
#        #        try:
#        #            routing_number.validate()
#        #        except balanced.exc.HTTPError as ex:
#        #            if 'routing_number' in ex.message:
#        #                raise forms.ValidationError(ex.message)
#        #            raise
#        return data


class BankAccountAdmin(BalancedAdmin):
    add_fields = ('name', 'account_number', 'routing_number', 'type', 'user')
    edit_fields = ('user',)
    list_display = ['account_number', 'created_at', 'user', 'name',
                    'bank_name', 'type', 'dashboard_link']
    list_filter = ['type', 'bank_name', 'user']
    search_fields = ['name', 'account_number']
    form = BankAccountAdminForm
    actions = ['bulk_pay_action']

    def bulk_pay_action(self, request, queryset):
        return render(request, 'django_balanced/admin_confirm_bulk_pay.html', {
            'bank_accounts': enumerate(queryset),
        }, current_app=self.admin_site.name)

    bulk_pay_action.short_description = 'Credit selected accounts'

    def get_urls(self):
        urls = super(BankAccountAdmin, self).get_urls()
        my_urls = patterns('django_balanced.admin',
                           url(r'^bulk_pay/$',
                               self.admin_site.admin_view(self.bulk_pay_view),
                               name='bank_account_bulk_pay')
                           )
        return my_urls + urls

    def bulk_pay_view(self, request):
        charges = []
        index = 0
        total = 0
        while True:
            prefix = 'bank_account_%s' % index
            uri = request.POST.get(prefix)
            if not uri:
                break
            description = request.POST.get('%s_description' % prefix)
            amount = float(request.POST.get('%s_amount' % prefix))
            amount = int(amount * 100)
            bank_account = BankAccount.objects.get(pk=uri)
            total += amount
            charges.append(
                (bank_account, amount, description)
            )
            index += 1
        balanced.bust_cache()
        escrow = balanced.Marketplace.my_marketplace.in_escrow
        if total > escrow:
            raise Exception('You have insufficient funds.')
        for bank_account, amount, description in charges:
            bank_account.credit(amount, description)
        return redirect(urlresolvers.reverse('admin:index'))

    def save_model(self, request, obj, form, change):
        data = form.data
        obj.name = data['name']
        obj.account_number = data['account_number']
        obj.routing_number = data['routing_number']
        obj.type = data['type']
        if data['user']:
            obj.user = User.objects.get(pk=data['user'])
        super(BalancedAdmin, self).save_model(request, obj, form, change)


class CreditAdminForm(forms.ModelForm):
    amount = forms.DecimalField(max_digits=10, required=True)
    description = forms.CharField(max_length=255, required=False)
    bank_account = forms.ModelChoiceField(queryset=BankAccount.objects)

    class Meta:
        model = Credit

    def clean(self):
        if not self.is_valid():
            return self.cleaned_data
        data = self.cleaned_data
        balanced.bust_cache()
        escrow = balanced.Marketplace.my_marketplace.in_escrow
        amount = int(float(data['amount']) * 100)
        if amount > escrow:
            raise forms.ValidationError('You have insufficient funds to cover '
                                        'this transfer.')
        return data


class CreditAdmin(BalancedAdmin):
    add_fields = ('amount', 'bank_account', 'description')
    edit_fields = (None)
    list_display = ['user', 'bank_account', 'amount',
                    'description', 'status', 'dashboard_link']
    search_fields = ['amount', 'description', 'status']
    list_filter = ['user', 'status']
    form = CreditAdminForm

    def get_form(self, request, obj=None, **kwargs):
        if obj:
            self.exclude = ('amount',)
        return super(CreditAdmin, self).get_form(request, obj=None, **kwargs)

    def save_model(self, request, obj, form, change):
        data = form.data
        amount = int(float(data['amount']) * 100)
        bank_account = BankAccount.objects.get(pk=data['bank_account'])
        obj.amount = amount
        obj.bank_account = bank_account
        obj.user = bank_account.user
        obj.description = data['description']
        obj.save()


admin.site.register(BankAccount, BankAccountAdmin)
admin.site.register(Credit, CreditAdmin)



from __future__ import unicode_literals

import balanced
import mock

from django.test import TestCase
from django.contrib.auth.models import User

from django_balanced import models


# https://www.balancedpayments.com/docs/testing
FIXTURES = {
    'card': {
        'card_number': 4111111111111111,
        'expiration_month': 12,
        'expiration_year': 2020,
    },
    'bank_account': {
        'account_number': 123123123,
        'routing_number': 321174851,  # SMCU
        'type': 'savings',
        'name': 'dan carter',
    }
}


class ModelsTest(TestCase):

    @classmethod
    def setUpClass(cls):
        with balanced.key_switcher(None):
            cls.api_key = balanced.APIKey().save()
        balanced.configure(cls.api_key.secret)
        cls.marketplace = balanced.Marketplace().save()
        cls.user = User.objects.create_user('john', 'john@test.com', 'pass')
        cls.user.save()

        card = balanced.Card(**FIXTURES['card']).save()
        cls.card = models.Card.create_from_card_uri(cls.user, card.uri)
        cls.buyer = cls.card.user.balanced_account
        # put some money in the escrow account
        cls.buyer.debit(100 * 100, 'test')  # $100.00

    def setUp(self):
        pass

    def test_create_credit(self):
        bank_account = models.BankAccount(**FIXTURES['bank_account'])
        bank_account.save()

        # create a second bank account to ensure that we're testing the correct
        # bank account in the case of multiple accounts being available.
        models.BankAccount(**FIXTURES['bank_account']).save()
        credit = bank_account.credit(100)
        self.assertEqual(credit.amount * 100, 100)
        self.assertEqual(credit.bank_account.uri, bank_account.uri)

    def test_create_bank_account(self):
        bank_account = models.BankAccount(**FIXTURES['bank_account'])
        bank_account.save()
        self.assertTrue(bank_account.bank_name)
        self.assertTrue(bank_account.uri)



"""
Balanced Python client library.

See ``README.mkd`` for usage advice.
"""
import os
import re
from distutils.core import Command

try:
    import setuptools
except ImportError:
    import distutils.core

    setup = distutils.core.setup
else:
    setup = setuptools.setup


def _get_version():
    path = os.path.join(PATH_TO_FILE, 'django_balanced', '__init__.py')
    version_re = r".*__version__ = '(.*?)'"
    fo = open(path)
    try:
        return re.compile(version_re, re.S).match(fo.read()).group(1)
    finally:
        fo.close()


def _get_long_description():
    path = os.path.join(PATH_TO_FILE, 'README.md')
    fo = open(path)
    try:
        return fo.read()
    finally:
        fo.close()


def parse_requirements(file_name):
    requirements = []
    for line in open(file_name, 'r').read().split('\n'):
        if re.match(r'(\s*#)|(\s*$)', line):
            continue
        if re.match(r'\s*-e\s+', line):
            requirements.append(re.sub(r'\s*-e\s+.*#egg=(.*)$', r'\1', line))
        elif re.match(r'\s*-f\s+', line):
            pass
        else:
            requirements.append(line)

    return requirements


def parse_dependency_links(file_name):
    dependency_links = []
    for line in open(file_name, 'r').read().split('\n'):
        if re.match(r'\s*-[ef]\s+', line):
            dependency_links.append(re.sub(r'\s*-[ef]\s+', '', line))

    return dependency_links


PATH_TO_FILE = os.path.dirname(__file__)
VERSION = _get_version()
LONG_DESCRIPTION = _get_long_description()


setup(
    name='django-balanced',
    version=VERSION,
    url='https://balancedpayments.com/',
    license='BSD',
    author='Balanced',
    author_email='support@balancedpayments.com',
    description='Payments platform for marketplaces',
    long_description=LONG_DESCRIPTION,
    packages=[
        'django_balanced',
        'django_balanced.management',
        'django_balanced.management.commands',
    ],
    package_dir={
        'django_balanced': 'django_balanced',
    },
    package_data={
        '': ['README.md'],
    },
    data_files={
        'templates': ['django_balanced/templates/django_balanced/admin_confirm_bulk_pay.html'],
    },
    test_suite='nose.collector',
    install_requires=parse_requirements('requirements.txt'),
    dependency_links=parse_dependency_links('requirements.txt'),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
